package com.gx.obe.template.agent;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.template.entity.ProjectTemplate;
import com.gx.obe.template.service.ProjectTemplateService;
import com.gx.obe.template.vo.ProjectTypeVo;

/**
 * @Description: 项目模板代理
 * @author chenxw
 */
public class ProjectTemplateAgent {
	
	private final ProjectTemplateService PTS = new ProjectTemplateService();
	
	private Optional<Stream<ProjectTemplate>> projectTemplateStreamOptional;
	
	public ProjectTemplateAgent(Function<ProjectTemplateService, List<ProjectTemplate>> function) {
		projectTemplateStreamOptional = Optional.of(PTS).map(function).map(List::stream);
	}

	/**
	 * @Description: 初始化全部列表
	 * @author chenxw
	 * @return
	 */
	public static ProjectTemplateAgent initAllList() {
		return new ProjectTemplateAgent(ProjectTemplateService::getAllList);
	}
	
	/**
	 * @Description: 初始化根据项目加载列表
	 * @author chenxw
	 * @param tenderId 
	 * @param templateType 
	 * @return
	 */
	public static ProjectTemplateAgent initProjectList(String tenderId, String templateType) {
		return new ProjectTemplateAgent(s -> s.getListByTenderIdAndTemplateType(tenderId, templateType));
	}
	
	/**
     * @param function
     * @param predicate
     * @return
     * @Description: 添加过滤器
     * @author chenxw
     */
    public <T> ProjectTemplateAgent filterType(Function<ProjectTypeVo, T> function, Predicate<T> predicate) {
        return filterType(t -> predicate.test(function.apply(t)));
    }

    /**
     * @param predicate
     * @return
     * @Description: 添加过滤器
     * @author chenxw
     */
    public ProjectTemplateAgent filterType(Predicate<ProjectTypeVo> predicate) {
        return filter(t -> predicate.test(getProjectType(t.getProjectType())));
    }
	
	/**
	 * @Description: 添加过滤器
	 * @author chenxw
	 * @param function
	 * @param predicate
	 * @return
	 */
	public <T> ProjectTemplateAgent filter(Function<ProjectTemplate, T> function, Predicate<T> predicate) {
		return filter(t -> predicate.test(function.apply(t)));
	}
	
	/**
	 * @Description: 添加过滤器
	 * @author chenxw
	 * @param predicate
	 * @return 
	 */
	public ProjectTemplateAgent filter(Predicate<ProjectTemplate> predicate) {
		projectTemplateStreamOptional = projectTemplateStreamOptional.map(s -> s.filter(predicate));
		return this;
	}
	
	/**
	 * @Description: set
	 * @author chenxw
	 * @param action
	 * @return 
	 */
	public ProjectTemplateAgent peek(Consumer<ProjectTemplate> action) {
		projectTemplateStreamOptional = projectTemplateStreamOptional.map(s -> s.peek(action));
		return this;
	}
	
	/**
	 * @Description: 排序
	 * @author chenxw
	 * @param comparator
	 * @return 
	 */
	public ProjectTemplateAgent sorted(Comparator<ProjectTemplate> comparator) {
		projectTemplateStreamOptional = projectTemplateStreamOptional.map(s -> s.sorted(comparator));
		return this;
	}
	
	/**
	 * @Description: 获取过滤后的数据
	 * @author chenxw
	 * @return
	 */
	public List<ProjectTemplate> get() {
		return projectTemplateStreamOptional.map(s -> s.collect(Collectors.toList())).orElseGet(ArrayList::new);
	}
	
	/**
	 * @Description: 遍历过滤后的数据
	 * @author chenxw
	 * @param action
	 */
	public void forEach(Consumer<ProjectTemplate> action) {
		projectTemplateStreamOptional.ifPresent(s -> s.forEach(action));
	}
	
	/**
	 * @Description: json转对象
	 * @author chenxw
	 * @param projectType
	 * @return
	 */
	public static ProjectTypeVo getProjectType(String projectType) {
		return JsonUtil.jsonToBean(projectType, ProjectTypeVo.class);
	}
	
}
